/* This file is part of the wvWare 2 project
   Copyright (C) 2002 Werner Trobin <trobin@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "headers97.h"
#include "olestream.h"
#include <iostream>

using namespace wvWare;

namespace
{
    const int headerTypes = 6;
}

Headers97::Headers97( U32 fcPlcfhdd, U32 lcbPlcfhdd, OLEStreamReader* tableStream )
{
    if ( lcbPlcfhdd == 0 )
        return;

    tableStream->push();
#ifdef WV2_DEBUG_HEADERS
    std::cerr << "Headers97::Headers97(): fc=" << fcPlcfhdd << " lcb=" << lcbPlcfhdd << std::endl
              << "   there are " << lcbPlcfhdd / 4 - 2 << " headers" << std::endl;
#endif

    // remove later (do we want asserts in here???)
    if ( lcbPlcfhdd % 4 )
        std::cerr << "Bug: m_fib.lcbPlcfhdd % 4 != 0!" << std::endl;
    else if ( ( lcbPlcfhdd / 4 - 2 ) % headerTypes )
        std::cerr << "Bug: #headers % " << headerTypes << " != 0!" << std::endl;

    tableStream->seek( fcPlcfhdd, SEEK_SET );

    U32 i = 0;
    for ( ; i < headerTypes * sizeof( U32 ); i += sizeof( U32 ) )
        if ( tableStream->readU32() )
            std::cerr << "Bug: Read a value != 0 where I expected a 0!" << std::endl;
    for ( ; i < lcbPlcfhdd; i += sizeof( U32 ) )
        m_headers.push_back( tableStream->readU32() );

    tableStream->pop();
}

std::pair<U32, U32> Headers97::findHeader( int sectionNumber, unsigned char mask ) const
{
    U32 start = 0;
    U32 lim = 0;
    int offset = maskToOffset( mask );
    do {
        const int tmp = sectionNumber * headerTypes + offset;
        start = m_headers[ tmp ];
        lim = m_headers[ tmp + 1 ];
        --sectionNumber;
    } while ( start == lim && sectionNumber >= 0 );

    return std::make_pair( start, lim );
}

int Headers97::maskToOffset( unsigned char mask ) const
{
    int offset = 0;
    while ( mask != 0 && ( mask & 1 ) == 0 ) {
        ++offset;
        mask >>= 1;
    }
    return offset;
}
